/*
 *     Botendo
 *     Copyright (C) 2023  limited_dev
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package de.limited_dev.botendo.commands.slash.music

import de.limited_dev.botendo.Bot
import de.limited_dev.botendo.commands.slash.component.SlashCommand
import de.limited_dev.botendo.commands.slash.component.options.CommandOption
import de.limited_dev.botendo.util.MessageUtil
import de.limited_dev.botendo.util.TimeUtil
import de.limited_dev.botendo.util.UrlUtil
import dev.kord.core.behavior.interaction.response.DeferredPublicMessageInteractionResponseBehavior
import dev.kord.core.entity.interaction.GuildChatInputCommandInteraction
import dev.schlaubi.lavakord.audio.Link

class NowPlayingCommand : SlashCommand("nowplaying", "Show what's currently playing", null) {
    override suspend fun onSlashCommand(
        interaction: GuildChatInputCommandInteraction,
        response: DeferredPublicMessageInteractionResponseBehavior,
        args: Map<CommandOption, String>
    ) {
        val guildId = interaction.guildId
        val link = Bot.lava.getLink(guildId.toString())
        val player = link.player
        if (link.state != Link.State.CONNECTED) {
            MessageUtil.sendEmbedForInteraction(
                interaction,
                response,
                "Not connected",
                "I'm not in a VC and therefore, I am not playing anything."
            )
            return
        }
        val track = player.playingTrack
        if (track == null) {
            MessageUtil.sendEmbedForInteraction(
                interaction,
                response,
                "Not Playing",
                "I'm not playing anything currently"
            )
            return
        }
        MessageUtil.sendEmbedForInteractionWithImage(
            interaction,
            response,
            "Currently playing",
            "**${track.title}**\n*Now Playing*\nby ${track.author} :: ${
                TimeUtil.getTimeFormatedRaw(
                    track.length.inWholeMilliseconds
                )
            }\n" +
                    ">>>${track.uri}",
            "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(track.uri!!) + "/maxresdefault.jpg"
        )
    }
}
