/*
 *     Botendo
 *     Copyright (C) 2023  limited_dev
 *
 *     This program is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package de.limited_dev.botendo.commands.slash.music

import de.limited_dev.botendo.Bot.lava
import de.limited_dev.botendo.commands.slash.component.SlashCommand
import de.limited_dev.botendo.commands.slash.component.options.CommandOption
import de.limited_dev.botendo.commands.slash.component.options.OptionType
import de.limited_dev.botendo.commands.slash.music.component.MusicManager
import de.limited_dev.botendo.util.MessageUtil
import dev.kord.core.behavior.interaction.response.DeferredPublicMessageInteractionResponseBehavior
import dev.kord.core.entity.interaction.GuildChatInputCommandInteraction
import dev.schlaubi.lavakord.audio.Link

class PlayCommand : SlashCommand(
    "play",
    "Play music",
    listOf(CommandOption("linkquery", "Either a search query or a Link", true, OptionType.STRING))
) {
    override suspend fun onSlashCommand(
        interaction: GuildChatInputCommandInteraction,
        response: DeferredPublicMessageInteractionResponseBehavior,
        args: Map<CommandOption, String>
    ) {
        val guildId = interaction.guildId
        val link = lava.getLink(guildId.toString())

        val voiceState = interaction.user.getVoiceStateOrNull()
        if (voiceState == null) {
            MessageUtil.sendEmbedForInteraction(
                interaction,
                response,
                "You are not connected to a VC",
                "Please connect to a VC"
            )
            return
        }

        val channelId = voiceState.channelId

        if (link.state != Link.State.CONNECTED) {
            link.connectAudio(channelId!!.value)
        } else if (link.state == Link.State.CONNECTED && link.lastChannelId != channelId!!.value) {
            MessageUtil.sendEmbedForInteraction(
                interaction,
                response,
                "You are not in my VC",
                "We are not in the same VC and therefore, you cannot play any music"
            )
            return
        }

        val query = args[this.options!![0]]
        val search = if (query!!.startsWith("http")) {
            query
        } else {
            "ytsearch:$query"
        }

        MusicManager.addToQueue(interaction, response, link, search)
    }
}
